@@ -35,21 +35,26 @@ KNOWN_KEYS = {"nelec", "spin", "hf_occ", "schedule", "maxiter",
               "extrapolation", "cached_contraction", "singlet_embedding", "normalize_mps",
               "dmrgfci", "mrci", "mrcis", "mrcisd", "mrcisdt", "casci", "nevpt2", "nevpt2s",
               "nevpt2sd", "nevpt2-ijrs", "nevpt2-ij", "nevpt2-rs", "nevpt2-ijr",
-              "nevpt2-rsi", "nevpt2-ir", "nevpt2-i", "nevpt2-r",
+              "nevpt2-rsi", "nevpt2-ir", "nevpt2-i", "nevpt2-r", "mrrept2", "mrrept2s",
+              "mrrept2sd", "mrrept2-ijrs", "mrrept2-ij", "mrrept2-rs", "mrrept2-ijr",
+              "mrrept2-rsi", "mrrept2-ir", "mrrept2-i", "mrrept2-r",
               "big_site", "stopt_dmrg", "stopt_compression", "stopt_sampling",
               "model", "k_symmetry", "k_irrep", "k_mod", "init_mps_center",
               "use_complex", "real_density_matrix", "expt_algo_type",
               "davidson_max_iter", "davidson_soft_max_iter", "one_body_parallel_rule",
               "n_sub_sweeps", "complex_mps", "split_states", "trans_mps_to_complex",
-              "use_general_spin", "trans_integral_to_spin_orbital", "store_wfn_spectra"}
+              "use_general_spin", "trans_integral_to_spin_orbital", "store_wfn_spectra",
+              "tran_bra_range", "tran_ket_range", "mem_ratio", "min_mpo_mem", "qc_mpo_type",
+              "full_integral"}
 
 REORDER_KEYS = {"noreorder", "fiedler", "reorder", "gaopt", "nofiedler",
                 "irrep_reorder"}
 
-DYN_CORR_KEYS = {"dmrgfci", "mrci", "mrcis", "mrcisd", "mrcisdt", "casci", "nevpt2",
-                 "nevpt2s", "nevpt2sd"}
+DYN_CORR_KEYS = {"dmrgfci", "mrci", "mrcis", "mrcisd", "mrcisdt", "casci", "mrrept2",
+                 "mrrept2s", "mrrept2sd"}
 
-NEVPT_KEYS = {"nevpt2", "nevpt2s", "nevpt2sd"}
+MRPT_KEYS = {"nevpt2", "nevpt2s", "nevpt2sd", "nevpt2-ijrs", "nevpt2-ij", "nevpt2-rs", "nevpt2-ijr",
+             "nevpt2-rsi", "nevpt2-ir", "nevpt2-i", "nevpt2-r"}
 
 RESTART_KEYS = {"restart_onepdm", "restart_twopdm", "restart_oh",
                 "restart_correlation", "restart_tran_onepdm", "restart_tran_twopdm",
@@ -234,7 +239,7 @@ def parse(fname):
         dic["restart_correlation"] = ""
     if len(set(dic.keys()) & RESTART_KEYS) != 0:
         dic["fullrestart"] = ""
-    if len(set(dic.keys()) & NEVPT_KEYS) != 0:
+    if len(set(dic.keys()) & MRPT_KEYS) != 0:
         dic["onepdm"] = ""
 
     return dic
@@ -337,8 +342,8 @@ def orbital_reorder(fcidump, method='gaopt'):
         midx = np.array(idx)
     elif method.startswith("manual "):
         method = method[len("manual "):]
-        idx = [int(x)
-               for x in open(method[len("manual "):], "r").readline().split()]
+        idx = [int(x)-1
+               for x in open(method, "r").readline().split()]
         f = OrbitalOrdering.evaluate(n_sites, kmat, VectorUInt16(idx))
         idx = np.array(idx)
         midx, mf = idx, f
